package stack;
/**
* Classe che implementa una struttura LIFO.
* Non accetta elementi nulli.
*/
public class Stack<T>
{
	private static final int maxSize = 5;

	private T[] storage = null;
	private int count = 0;

	public Stack(int max)
	{
		storage = (T[])new Object[max];
	}

	public Stack()
	{
		this(maxSize);
	}

	/**
	* Metodo di inserimento.
	* Ritorna true in caso di successo, false in caso di fallimento.
	* Non accetta elementi null.
	*/
	public boolean push(T elem)
	{
		if(elem == null)
		{
			System.err.println("Null is not acceptable");
			return false;
		}
		if(isFull())
		{
			System.err.println("Stack full");
			return false;
		}
		storage[count++] = elem;
		return true;
	}

	/**
	* Metodo di lettura distruttiva.
	* Ritorna null in caso di stack vuoto.
	*/
	public T pop()
	{
		if(isEmpty())
		{
			System.err.println("Stack empty");
			return null;
		}
		T retval = storage[--count];
		storage[count] = null;
		return retval;
	}

	/**
	* Metodo di interrogazione sul numero di elementi contenuti.
	*/
	public int size()
	{
		return count;
	}

	/**
	* Metodo di interrogazione sulla condizione di stack vuoto.
	*/
	public boolean isEmpty()
	{
		return size() == 0;
	}

	/**
	* Metodo di interrogazione sulla condizione di stack pieno.
	*/
	public boolean isFull()
	{
		return count == storage.length;
	}

	/**
	* Metodo per la rappresentazione stringa dello stack.
	*/
	public String toString()
	{
		String retval = "[";
		for(int i = count-1;i > 0;i--)
		{
			retval = retval.concat(storage[i].toString() + ", ");
		}
		if(storage[0] != null)
		{
			retval = retval.concat(storage[0].toString());
		}
		retval = retval.concat("]");
		return retval;
	}
}
